/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jmsconsumer;

import java.util.HashMap;
import java.util.Map;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.jms.HornetQJMSClient;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;
import org.hornetq.core.remoting.impl.netty.TransportConstants;
import org.hornetq.jms.client.HornetQConnectionFactory;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 *
 * @author hbragg
 */
public class JmsQ {
  private QueueConnection queueConnection = null;
  private QueueSession queueSession = null;
  private Queue queue = null;
  private QueueReceiver receiver = null;
  private static final Logger log = Logger.getLogger(JmsQ.class.getName());


  public JmsQ() {}

  /**
   * Start consuming messages.
   * @see javax.jms.QueueConnection.start(void)
   */
  public void startReceiver() throws JMSException {
    queueConnection.start();
}

  /**
   * Commit received messages transaction.
   * @see javax.jms.QueueSession.commit(void)
   */
  public void commit() throws JMSException {
    queueSession.commit();
  }

  /**
   * Rollback received messages transaction.
   * @see javax.jms.QueueSession.rollback(void)
   */
  public void rollback() throws JMSException {
    queueSession.rollback();
  }

  /**
   * Asynchronously received messages.
   * @see javax.jms.MessageConsumer.receiveNoWait(void)
   */
  public Message receiveNoWait() throws JMSException {
    return receiver.receiveNoWait();
  }

  /**
   * Synchronous received messages.
   * @see javax.jms.MessageConsumer.receive(void)
   */
  public Message receive() throws JMSException {
    return receiver.receive();
  }

  /**
   * Mostly Synchronous received messages with timeout.
   * @see javax.jms.MessageConsumer.receive(long)
   */
  public Message receive(long timeout) throws JMSException {
    return receiver.receive(timeout);
  }

  /**
   * Temporarily stop consuming messages.
   * @see javax.jms.QueueConnection.stop(void)
   */
  public void pauseReceiver() throws JMSException {
    queueConnection.stop();
  }

  /**
   * Stop and then restart the consumer.
   * normally used in the event of a failure.
   * @param cfg configuration object
   */
  public void restartReceiver() throws JMSException {
    StopReceiver();
    SetupReceiver();
    startReceiver();
  }

  /**
   * Stop listening to JMS queue. Disconnect from queue.
   */
  public void StopReceiver() {
    if (receiver != null) try {
      receiver.close();
      receiver = null;
    } catch (JMSException ex) {
      Logger.getLogger(Main.class.getName())
              .log(Level.SEVERE, null, ex);
    }
    if (queueConnection != null) try {
      queueConnection.stop();
    } catch (JMSException ex) {
      Logger.getLogger(Main.class.getName())
              .log(Level.SEVERE, null, ex);
    }
    if (queueSession != null) try {
      queueSession.close();
    } catch (JMSException ex) {
      Logger.getLogger(Main.class.getName())
              .log(Level.SEVERE, null, ex);
    } finally {
      queueSession = null;
    }
    if (queueConnection != null) try {
      queueConnection.close();
    } catch (JMSException ex) {
      Logger.getLogger(Main.class.getName())
              .log(Level.SEVERE, null, ex);
    } finally {
      queueConnection = null;
    }
  }

  /**
   * Connect to JMS queue and create a consumer
   * @param cfg configuration object
   */
  public void SetupReceiver() throws JMSException {
    String queueName = "databus";
    String host = "192.168.196.110";
    String port = "5445";
    log.log(Level.INFO, "Attempting to Connect to JMS to queue {0} on {1}:{2}", new Object[]{queueName, host, port});

    // Step 1. Directly instantiate the JMS Queue object.
    queue = HornetQJMSClient.createQueue(queueName);

    // Step 2. Instantiate the TransportConfiguration object which contains
    // the knowledge of what transport to use,
    Map<String, Object> connectionParams = new HashMap<String, Object>();
    connectionParams.put(TransportConstants.PORT_PROP_NAME, port);
    connectionParams.put(TransportConstants.HOST_PROP_NAME, host);

    TransportConfiguration transportConfiguration =
        new TransportConfiguration(
             NettyConnectorFactory.class.getName(),
             connectionParams);
    // Step 3 Directly instantiate the JMS ConnectionFactory
    // object using that TransportConfiguration
    HornetQConnectionFactory hqcf = HornetQJMSClient
            .createConnectionFactory(transportConfiguration);
    hqcf.setRetryInterval(2000);          // 2 seconds for first retry
    hqcf.setRetryIntervalMultiplier(1.5); // 1.5 times loner betrween retrys
    hqcf.setMaxRetryInterval(20000);      // Wait max 20 secs between retrys
    hqcf.setReconnectAttempts(-1);        // Retry forever


    // Step 4.Create a JMS Connection
    queueConnection = (QueueConnection)hqcf.createConnection();

    // Step 5. Create a JMS Session
    queueSession = (QueueSession)queueConnection
            .createSession(true, Session.AUTO_ACKNOWLEDGE);

    // Step 6. Create the consumer
    receiver = queueSession.createReceiver(queue);

    log.info("JMS Connection established.");
  }

  // Just a test interface
  public void Send(String text) throws JMSException {
    QueueSender sender = queueSession.createSender(queue);
    TextMessage tm = queueSession.createTextMessage(text);
    sender.send(tm);
    tm.acknowledge();
    sender.close();
  }
}
